==> ciao-4.6-src-core.tar.gz: ciao-4.6/src/lib/help/AhelpDirectory.cc <==
// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <regex.h>
#include <string.h>

#include "Ahelpstd_types.h"

#include "AhelpDirectory.hh"


// Constructor
AhelpDirectory::AhelpDirectory() : AhelpStringsList()
{
  _regexp = NULL;
}

AhelpDirectory::AhelpDirectory(char *fname,DirMode mode,Boolean dotfiles) : AhelpStringsList()
{
  _regexp = NULL;
  ReadDir(fname,mode,dotfiles);
}

AhelpDirectory::AhelpDirectory(char *fname,char *regexp, DirMode mode, 
					Boolean dotfiles) : AhelpStringsList()
{
   // The regexp s/b in the form of an egrep-style extended regular
   // expression.  For example, to list only *.fits files, pass
   // ".*\\.fits".  To list *.cc and *.hh files, for example, pass 
   // ".*\\.((cc)|(hh))".

  _regexp = regexp;
  ReadDir(fname,mode,dotfiles);
}

AhelpDirectory::~AhelpDirectory() 
{ 
}

Boolean AhelpDirectory::ReadDir(char *fname,DirMode mode,Boolean dotfiles) 
{
  AhelpStrings       filename;
  DIR           *dir;
  struct dirent *file;
  Boolean       return_stat = FALSE;
  Boolean       filestat;
  struct stat   stat_buf;
  int           amode;
  regex_t  	regExp;
  int 		match=1;

  if (_regexp)
     if ((amode=regcomp(&regExp,_regexp,REG_EXTENDED|REG_NOSUB)) != 0)
	return return_stat;

  if ((dir = opendir(fname)) != NULL)
  {

    while ((file = readdir(dir)) != NULL)
    {

      if (_regexp)
      	 match = !regexec(&regExp,file->d_name,0,NULL,0);

      filestat = FALSE;
      if (match && (dotfiles ||
      		   (strcmp(file->d_name,".") && strcmp(file->d_name,"..")))
	 )
        {
        if (mode != NO_MODE)
          {
          filename = fname;
          filename.append("/");
          filename.append(file->d_name);
          if (stat(filename.chars(),&stat_buf) == 0)
            {
            if (mode == DIR_MODE)
              {
              if (stat(filename.chars(),&stat_buf) == 0)
                if (S_ISDIR(stat_buf.st_mode))
                  filestat = TRUE;
              }
            else if (!(S_ISDIR(stat_buf.st_mode)))
              {
              if (mode == W_MODE)
                amode = W_OK;
              else if(mode==X_MODE) 
                amode = X_OK;
              else if(mode==R_MODE) 
                amode = R_OK;
              else
                amode = F_OK;  // test for existence
              if (access(filename.chars(),amode) == 0)
                filestat = TRUE;
              }
            }
          }
        else
          filestat = TRUE;
        if (filestat)
          {
	  filename = file->d_name;
          this->Append(filename);
          }
        }
      }

    return_stat = TRUE;
    closedir(dir);
  }

  if (_regexp)
      regfree(&regExp);

  return return_stat;
}

==> ciao-4.8-src-core.tar.gz: ciao-4.8/src/lib/help/AhelpDirectory.cc <==
// 
//  Copyright (C) 2007,2015  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <regex.h>
#include <string.h>

#include "Ahelpstd_types.h"

#include "AhelpDirectory.hh"


// Constructor
AhelpDirectory::AhelpDirectory() : AhelpStringsList()
{
  _regexp = NULL;
}

AhelpDirectory::AhelpDirectory(char *fname,DirMode mode,Boolean dotfiles) : AhelpStringsList()
{
  _regexp = NULL;
  ReadDir(fname,mode,dotfiles);
}

AhelpDirectory::AhelpDirectory(char *fname, const char *regexp, DirMode mode, 
					Boolean dotfiles) : AhelpStringsList()
{
   // The regexp s/b in the form of an egrep-style extended regular
   // expression.  For example, to list only *.fits files, pass
   // ".*\\.fits".  To list *.cc and *.hh files, for example, pass 
   // ".*\\.((cc)|(hh))".

  if (regexp)
  {
     _regexp = (char*)malloc(strlen(regexp)+1);
     strcpy(_regexp, regexp);
  }
  ReadDir(fname,mode,dotfiles);
}

AhelpDirectory::~AhelpDirectory() 
{ 
   if (_regexp)
   {
      free(_regexp);
      _regexp = NULL;
   }
}

Boolean AhelpDirectory::ReadDir(char *fname,DirMode mode,Boolean dotfiles) 
{
  AhelpStrings       filename;
  DIR           *dir;
  struct dirent *file;
  Boolean       return_stat = FALSE;
  Boolean       filestat;
  struct stat   stat_buf;
  int           amode;
  regex_t  	regExp;
  int 		match=1;

  if (_regexp)
     if ((amode=regcomp(&regExp,_regexp,REG_EXTENDED|REG_NOSUB)) != 0)
	return return_stat;

  if ((dir = opendir(fname)) != NULL)
  {

    while ((file = readdir(dir)) != NULL)
    {

      if (_regexp)
      	 match = !regexec(&regExp,file->d_name,0,NULL,0);

      filestat = FALSE;
      if (match && (dotfiles ||
      		   (strcmp(file->d_name,".") && strcmp(file->d_name,"..")))
	 )
        {
        if (mode != NO_MODE)
          {
          filename = fname;
          filename.append("/");
          filename.append(file->d_name);
          if (stat(filename.chars(),&stat_buf) == 0)
            {
            if (mode == DIR_MODE)
              {
              if (stat(filename.chars(),&stat_buf) == 0)
                if (S_ISDIR(stat_buf.st_mode))
                  filestat = TRUE;
              }
            else if (!(S_ISDIR(stat_buf.st_mode)))
              {
              if (mode == W_MODE)
                amode = W_OK;
              else if(mode==X_MODE) 
                amode = X_OK;
              else if(mode==R_MODE) 
                amode = R_OK;
              else
                amode = F_OK;  // test for existence
              if (access(filename.chars(),amode) == 0)
                filestat = TRUE;
              }
            }
          }
        else
          filestat = TRUE;
        if (filestat)
          {
	  filename = file->d_name;
          this->Append(filename);
          }
        }
      }

    return_stat = TRUE;
    closedir(dir);
  }

  if (_regexp)
      regfree(&regExp);

  return return_stat;
}
